from datetime import datetime

from celery import shared_task
from django.conf import settings
from django.db import transaction
from django.db.models import Case, Count, ExpressionWrapper, F, Q, When
from django.utils import timezone
from dynamic_preferences.registries import global_preferences_registry

from shiftregister.app.models import ShiftRegistration

from .models import FallbackAssignment

global_preferences = global_preferences_registry.manager()


# cron task to send normal messages(reminders,changes) in batches
@shared_task
def deactivate_fallbacks():
    for f in FallbackAssignment.objects.annotate(
        real_required_helpers=Case(
            When(shift__required_helpers=0, then=F("shift__room__required_helpers")),
            default=F("shift__required_helpers"),
        ),
        reg_count=Count(
            "shift__shiftregistration",
            distinct=True,
            filter=Q(
                shift__shiftregistration__state__in=(
                    ShiftRegistration.RegState.REGISTERED,
                    ShiftRegistration.RegState.CHECKED_IN,
                )
            ),
        ),
    ).filter(
        was_full=False,
        shift__start_at__lte=timezone.now()
        + global_preferences["helper__min_cancel_time"],
        real_required_helpers__lte=F("reg_count"),
    ):
        f.was_full = True
        f.save()
